#pragma once

# ifdef KEYISO_TEST_WINDOWS
typedef unsigned char uuid_t[16];
# else
#   include <uuid/uuid.h>
# endif

# include <openssl/x509.h>

#ifdef  __cplusplus
extern "C" {
#endif


// Returns 1 for success and 0 for an error
int KeyIso_create_self_sign_pfx(
    const uuid_t correlationId,
    int keyisoFlags,
    const char *confStr,
    int *pfxLength,
    unsigned char **pfxBytes,           // KeyIso_clear_free()
    char **salt);                       // KeyIso_clear_free_string()

// Returns 1 for success and 0 for an error
int KeyIso_create_self_sign_pfx_to_key_id(
    const uuid_t correlationId,
    int keyisoFlags,
    const char *confStr,
    char **keyId);                    // KeyIso_clear_free_string()

// Returns 1 for success and 0 for an error
// The pemCert consists of the end certificate followed by
// 1 or more CA certificates
int KeyIso_replace_pfx_certs(
    const uuid_t correlationId,
    int keyisoFlags,
    int inPfxLength,
    const unsigned char *inPfxBytes,
    const char *inSalt,
    int pemCertLength,
    const unsigned char *pemCertBytes,
    int *outPfxLength,
    unsigned char **outPfxBytes,        // KeyIso_clear_free()
    char **outSalt);                    // KeyIso_clear_free_string()

// Returns 1 for success and 0 for an error
int KeyIso_replace_key_id_certs(
    const uuid_t correlationId,
    int keyisoFlags,
    const char *inKeyId,
    int pemCertLength,
    const unsigned char *pemCertBytes,
    char **outKeyId);                   // KeyIso_clear_free_string()

// Returns 1 for success and 0 for an error
int KeyIso_replace_key_id_certs2(
    const uuid_t correlationId,
    int keyisoFlags,
    const char *inKeyId,
    X509 *cert,
    STACK_OF(X509) *ca,                 // Optional
    char **outKeyId);                   // KeyIso_clear_free_string()


#ifdef  __cplusplus
}
#endif