#pragma once


# ifdef KEYISO_TEST_WINDOWS
typedef unsigned char uuid_t[16];
# else
#   include <uuid/uuid.h>
# endif

#include <openssl/evp.h>

#ifdef  __cplusplus
extern "C" {
#endif


// Return:
//  +1 - Success with complete chain of certificates to a trusted root
//  -1 - Success with chain error. Might be missing intermediate certs.
//       *verifyChainError is updated with X509_V_ERR_* error defined
//       in x509_vfy.h.
//   0 - Error, unable to import PFX.
int KeyIso_import_pfx_from_pem(
    const uuid_t correlationId,
    int keyisoFlags,
    int inKeyLength,
    const unsigned char *inKeyBytes,
    int inCertLength,
    const unsigned char *inCertBytes,
    const char *password,             			// Optional
    int *verifyChainError,
    int *outPfxLength,
    unsigned char **outPfxBytes,      			// KeyIso_free()
    char **salt);

// Return:
//  +1 - Success with complete chain of certificates to a trusted root
//  -1 - Success with chain error. Might be missing intermediate certs.
//       *verifyChainError is updated with X509_V_ERR_* error defined
//       in x509_vfy.h.
//   0 - Error, unable to import PFX.
int KeyIso_import_pfx_from_pem_to_key_id(
    const uuid_t correlationId,
    int keyisoFlags,
    int inKeyLength,
    const unsigned char *inKeyBytes,
    int inCertLength,
    const unsigned char *inCertBytes,
    const char *password,             			// Optional
    int *verifyChainError,
    char **keyId);                     			// KeyIso_free()

// Return:
//  +1 - Success with complete chain of certificates to a trusted root
//  -1 - Success with chain error. Might be missing intermediate certs.
//       *verifyChainError is updated with X509_V_ERR_* error defined
//       in x509_vfy.h.
//   0 - Error, unable to import PFX.
int KeyIso_import_pfx_from_pem_container_aware(
    const uuid_t correlationId,
    int keyisoFlags,
    int inKeyLength,
    const unsigned char *inKeyBytes,
    int inCertLength,
    const unsigned char *inCertBytes,
    const char *password,                       // Optional
    const unsigned char* containerAwareData,    // For example, Pod Unique ID 
    const unsigned int containerAwareDataLen,
    int *verifyChainError,
    int *outPfxLength,
    unsigned char **outPfxBytes,                // KeyIso_free()
    char **salt);

// Return:
//  +1 - Success with complete chain of certificates to a trusted root
//  -1 - Success with chain error. Might be missing intermediate certs.
//       *verifyChainError is updated with X509_V_ERR_* error defined
//       in x509_vfy.h.
//   0 - Error, unable to import PFX.
int KeyIso_import_pfx_from_pem_to_key_id_container_aware(
    const uuid_t correlationId,
    int keyisoFlags,
    int inKeyLength,
    const unsigned char *inKeyBytes,
    int inCertLength,
    const unsigned char *inCertBytes,
    const char *password,                       // Optional
    const unsigned char* containerAwareData,    // For example, Pod Unique ID 
    const unsigned int containerAwareDataLen,
    int *verifyChainError,
    char **keyId);                               // KeyIso_free()

// Return:
//  +1 - Success with complete chain of certificates to a trusted root
//  -1 - Success with chain error. Might be missing intermediate certs.
//       *verifyChainError is updated with X509_V_ERR_* error defined
//       in x509_vfy.h.
//   0 - Error, unable to import PFX.
int KeyIso_import_pfx(
    const uuid_t correlationId,
    int keyisoFlags,
    int inPfxLength,
    const unsigned char *inPfxBytes,
    const char *password,             // Optional
    int *verifyChainError,
    int *pfxLength,
    unsigned char **pfxBytes,         // KeyIso_clear_free()
    char **salt);                     // KeyIso_clear_free_string()

// Return:
//  +1 - Success with complete chain of certificates to a trusted root
//  -1 - Success with chain error. Might be missing intermediate certs.
//       *verifyChainError is updated with X509_V_ERR_* error defined
//       in x509_vfy.h.
//   0 - Error, unable to import PFX.
int KeyIso_import_pfx_to_key_id(
    const uuid_t correlationId,
    int keyisoFlags,
    int inPfxLength,
    const unsigned char *inPfxBytes,
    const char *password,             // Optional
    int *verifyChainError,
    char **keyId);                    // KeyIso_clear_free_string()

// Return:
//  +1 - Success with complete chain of certificates to a trusted root
//  -1 - Success with chain error. Might be missing intermediate certs.
//       *verifyChainError is updated with X509_V_ERR_* error defined
//       in x509_vfy.h.
//   0 - Error, unable to import PFX.
int KeyIso_import_pfx_container_aware(
    const uuid_t correlationId,
    int keyisoFlags,
    int inPfxLength,
    const unsigned char* inPfxBytes,
    const char* password,             // Optional
    const unsigned char* containerAwareData,   // For example, Pod Unique ID 
    const unsigned int containerAwareDataLen,
    int* verifyChainError,
    int* pfxLength,
    unsigned char** pfxBytes,         // KeyIso_clear_free()
    char** salt);                     // KeyIso_clear_free_string()

// Return:
//  +1 - Success with complete chain of certificates to a trusted root
//  -1 - Success with chain error. Might be missing intermediate certs.
//       *verifyChainError is updated with X509_V_ERR_* error defined
//       in x509_vfy.h.
//   0 - Error, unable to import PFX.
int KeyIso_import_pfx_to_key_id_container_aware(
    const uuid_t correlationId,
    int keyisoFlags,
    int inPfxLength,
    const unsigned char* inPfxBytes,
    const char* password,             // Optional
    const unsigned char* containerAwareData,   // For example, Pod Unique ID 
    const unsigned int containerAwareDataLen,
    int* verifyChainError,
    char** keyId);                    // KeyIso_clear_free_string()

int KeyIso_format_pfx_engine_key_id(
    const uuid_t correlationId,
    int pfxLength,
    const unsigned char *pfxBytes,
    const char *salt,
    char **keyId);                    // KeyIso_clear_free_string()


int KeyIso_parse_pfx_engine_key_id(
    const uuid_t correlationId,
    const char *keyId,
    int *pfxLength,
    unsigned char **pfxBytes,         // KeyIso_clear_free()
    char **salt);                     // Optional, KeyIso_clear_free_string()

// Return:
//  +1 - Success with complete chain of certificates to a trusted root
//  -1 - Success with chain error. Might be missing intermediate certs.
//       *verifyChainError is updated with X509_V_ERR_* error defined
//       in x509_vfy.h.
//   0 - Error, unable to import PFX.
int KeyIso_build_cert_chain_from_pfx(
    const uuid_t correlationId,
    int keyisoFlags,
    int pfxLength,
    const unsigned char *pfxBytes,
    int *verifyChainError,
    int *pemCertLength,              // Excludes NULL terminator
    char **pemCert);                 // KeyIso_free()                   

// Return:
//  +1 - Success with complete chain of certificates to a trusted root
//  -1 - Success with chain error. Might be missing intermediate certs.
//       *verifyChainError is updated with X509_V_ERR_* error defined
//       in x509_vfy.h.
//   0 - Error, unable to import PFX.
int KeyIso_build_cert_chain_from_key_id(
    const uuid_t correlationId,
    int keyisoFlags,
    const char *keyId,
    int *verifyChainError,
    int *pemCertLength,              // Excludes NULL terminator
    char **pemCert);                 // KeyIso_free()     


EVP_PKEY *KeyIso_load_engine_private_key(
    const uuid_t correlationId,
    const char *engineName,
    const char *engineKeyId);              


#ifdef  __cplusplus
}
#endif


