#ifndef HEADER_KMPP_GDBUS_CLIENT_H
# define HEADER_KMPP_GDBUS_CLIENT_H

#include "keyiso.h"
#include "keyisointernal.h"

#include <openssl/bio.h>
#include <gio/gio.h>

#ifdef  __cplusplus
extern "C" {
#endif



int KMPP_GDBUS_open_shared_mem(
    KEYISO_SHARED_MEM *sharedMem);

void KMPP_GDBUS_close_shared_mem(
    KEYISO_SHARED_MEM *sharedMem);

int KMPP_GDBUS_CLIENT_cert_ctrl(
    const uuid_t correlationId,
    KEYISO_SHARED_MEM *sharedMem,
    int ctrl,
    int location,
    int format,
    int length,
    const unsigned char *bytes);

int KMPP_GDBUS_CLIENT_import_pfx(
    const uuid_t correlationId,
    int keyisoFlags,
    int inPfxLength,
    const unsigned char *inPfxBytes,
    const char *inPassword,             // Optional
    const unsigned char* containerAwareData,   // For example, Pod Unique ID 
    const unsigned int containerAwareDataLen,
    int *outVerifyChainError,
    int *outPfxLength,
    unsigned char **outPfxBytes,        // KeyIso_free()
    char **outPfxSalt);                 // KeyIso_free()

int KMPP_GDBUS_CLIENT_pfx_open(
    KEYISO_PFX_CTX *keyCtx,
    int pfxLength,
    const unsigned char *pfxBytes,
    const char *salt);


void KMPP_GDBUS_CLIENT_pfx_close(
    KEYISO_PFX_CTX *keyCtx);

int KMPP_GDBUS_CLIENT_get_version(
    unsigned int *outVersion);

int KMPP_GDBUS_CLIENT_rsa_private_encrypt_decrypt(
    KEYISO_PFX_CTX *keyCtx,
    int decrypt,
    int flen,
    const unsigned char *from,
    int tlen,
    unsigned char *to,
    int padding);

int KMPP_GDBUS_CLIENT_ecdsa_sign(
    KEYISO_PFX_CTX *keyCtx,
    int type,
    const unsigned char *dgst,
    int dlen,
    unsigned char *sig,
    unsigned int siglen,
    unsigned int *outlen);

int KMPP_GDBUS_CLIENT_create_self_sign_pfx(
    const uuid_t correlationId,
    int keyisoFlags,
    const char *confStr,
    int *outPfxLength,
    unsigned char **outPfxBytes,        // KeyIso_free()
    char **outPfxSalt);                 // KeyIso_free()

// The pemCert consists of the end certificate followed by
// 1 or more CA certificates
int KMPP_GDBUS_CLIENT_replace_pfx_certs(
    const uuid_t correlationId,
    int keyisoFlags,
    int inPfxLength,
    const unsigned char *inPfxBytes,
    const char *inSalt,
    int pemCertLength,
    const unsigned char *pemCertBytes,
    int *outPfxLength,
    unsigned char **outPfxBytes,    // KeyIso_free()
    char **outSalt);                // KeyIso_free()

// Before returning
//  g_error_free(*error)
//  *error = NULL;
void _KMPP_GDBUS_trace_log_glib_error_para(
    const char *file,
    const char *func,
    const int line,
    const uuid_t correlationId,
    const int flags,
    const char *title,
    const char *loc,
    GError **error,           // Before returning, g_error_free(*error)
    const char *format, ...);
#define KMPP_GDBUS_trace_log_glib_error_para(correlationId, flags, title, loc, error, ...) \
    _KMPP_GDBUS_trace_log_glib_error_para(__FILE__, __FUNCTION__, __LINE__, correlationId, flags, title, loc, error, __VA_ARGS__)

// Before returning
//  g_error_free(*error)
//  *error = NULL;
void _KMPP_GDBUS_trace_log_glib_error(
    const char *file,
    const char *func,
    const int line,
    const uuid_t correlationId,
    const int flags,
    const char *title,
    const char *loc,
    GError **error);          // Before returning, g_error_free(*error)
#define KMPP_GDBUS_trace_log_glib_error(correlationId, flags, title, loc, error) \
    _KMPP_GDBUS_trace_log_glib_error(__FILE__, __FUNCTION__, __LINE__, correlationId, flags, title, loc, error)


#ifdef  __cplusplus
}
#endif

#endif /* HEADER_KMPP_GDBUS_CLIENT_H */
